"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var client_shape_1 = require("../client-shape");
var component_class_1 = require("../component-class");
var hoistStatics = require('hoist-non-react-statics');
var EMPTY_OBJECT = {};
var EMPTY_ARRAY = [];
function connectErrors(options) {
    if (options === void 0) { options = EMPTY_OBJECT; }
    return function (WrappedComponent) {
        var key = 0;
        var Connect = /** @class */ (function (_super) {
            __extends(Connect, _super);
            function Connect(props, context) {
                var _this = _super.call(this, props, context) || this;
                _this._onNetworkError = function (err) {
                    if (options.ignoreNetworkErrors) {
                        return;
                    }
                    err.key = 'network_error_key_' + (key++);
                    _this.setState({ networkErrors: _this.state.networkErrors.concat([err]) });
                };
                _this._onMutationError = function (err) {
                    if (options.ignoreMutationErrors) {
                        return;
                    }
                    err.key = 'mutation_error_key_' + (key++);
                    _this.setState({ mutationErrors: _this.state.mutationErrors.concat([err]) });
                };
                _this._onQueueRequest = function () {
                    _this.setState({ requestInFlight: true });
                };
                _this._onSuccessfulResponse = function (pendingMutations) {
                    _this.setState({
                        networkErrors: EMPTY_ARRAY,
                        requestInFlight: pendingMutations !== 0,
                    });
                };
                _this._dismissNetworkError = function (_a) {
                    var key = _a.key;
                    var networkErrors = _this.state.networkErrors.filter(function (err) { return err.key !== key; });
                    _this.setState({
                        networkErrors: networkErrors.length ? networkErrors : EMPTY_ARRAY,
                    });
                };
                _this._dismissMutationError = function (_a) {
                    var key = _a.key;
                    var mutationErrors = _this.state.mutationErrors.filter(function (err) { return err.key !== key; });
                    _this.setState({
                        mutationErrors: mutationErrors.length ? mutationErrors : EMPTY_ARRAY,
                    });
                };
                _this._client = props.client || context.bicycleClient;
                if (!_this._client) {
                    throw new Error("Could not find \"client\" in either the context or " +
                        ("props of \"" + Connect.displayName + "\". ") +
                        "Either wrap the root component in a <Provider>, " +
                        ("or explicitly pass \"client\" as a prop to \"" + Connect.displayName + "\"."));
                }
                _this.state = { networkErrors: [], mutationErrors: [], requestInFlight: false };
                return _this;
            }
            Connect.prototype.componentDidMount = function () {
                this._networkErrorsSubscription = this._client.subscribeToNetworkErrors(this._onNetworkError);
                this._mutationErrorsSubscription = this._client.subscribeToMutationErrors(this._onMutationError);
                if (this._client.subscribeToQueueRequest && this._client.subscribeToSuccessfulResponse) {
                    this._queueRequestSubscription = this._client.subscribeToQueueRequest(this._onQueueRequest);
                    this._successfulResponseSubscription = this._client.subscribeToSuccessfulResponse(this._onSuccessfulResponse);
                }
            };
            Connect.prototype.componentWillUnmount = function () {
                if (this._networkErrorsSubscription) {
                    this._networkErrorsSubscription.unsubscribe();
                }
                if (this._mutationErrorsSubscription) {
                    this._mutationErrorsSubscription.unsubscribe();
                }
                if (this._queueRequestSubscription) {
                    this._queueRequestSubscription.unsubscribe();
                }
                if (this._successfulResponseSubscription) {
                    this._successfulResponseSubscription.unsubscribe();
                }
            };
            Connect.prototype.render = function () {
                return React.createElement(WrappedComponent, __assign({}, this.props, { networkErrors: this.state.networkErrors, mutationErrors: this.state.mutationErrors, requestInFlight: this.state.requestInFlight, onDismissNetworkError: this._dismissNetworkError, onDismissMutationError: this._dismissMutationError }));
            };
            Connect.displayName = "ConnectErrors(" + component_class_1.getDisplayName(WrappedComponent) + ")";
            Connect.WrappedComponent = WrappedComponent;
            Connect.contextTypes = {
                bicycleClient: client_shape_1.default,
            };
            return Connect;
        }(React.Component));
        return hoistStatics(Connect, WrappedComponent);
    };
}
exports.default = connectErrors;
//# sourceMappingURL=connect-errors.js.map