/// <reference types="react" />
import * as React from "react";
import * as PropTypes from "prop-types";
import { BaseRootQuery } from "bicycle/typed-helpers/query";
import BicycleClient, { QueryCacheResult } from "bicycle/client";
import ErrorResult from "bicycle/types/ErrorResult";
export { PropTypes };
export interface Props<QueryResult> {
    query: BaseRootQuery<QueryResult>;
    children: (result: QueryResult, client: BicycleClient, status: {
        result: QueryResult;
        loaded: boolean;
        errors: ReadonlyArray<string>;
        errorDetails: ReadonlyArray<ErrorResult>;
        previousElement: React.ReactNode;
        loadingDuration: number;
    }) => React.ReactNode;
    renderLoading?: boolean | ((loadingDuration: number, client: BicycleClient, status: {
        result: QueryResult;
        loaded: boolean;
        errors: ReadonlyArray<string>;
        errorDetails: ReadonlyArray<ErrorResult>;
        previousElement: React.ReactNode;
        loadingDuration: number;
    }) => React.ReactNode);
    renderErrors?: boolean | ((errors: ReadonlyArray<string>, client: BicycleClient, status: {
        result: QueryResult;
        loaded: boolean;
        errors: ReadonlyArray<string>;
        errorDetails: ReadonlyArray<ErrorResult>;
        previousElement: React.ReactNode;
        loadingDuration: number;
    }) => React.ReactNode);
}
export interface State<QueryResult> extends QueryCacheResult<QueryResult> {
    loadingDuration: number;
}
export default class Connect<QueryResult> extends React.Component<Props<QueryResult>, State<QueryResult>> {
    static contextTypes: {
        bicycleClient: PropTypes.Requireable<any>;
    };
    private _client;
    private _query;
    private _subscription;
    private _previousElement;
    _startLoadingTime: number | null;
    constructor(props: Props<QueryResult>, context: any);
    componentDidMount(): void;
    componentWillReceiveProps(nextProps: Props<QueryResult>): void;
    componentWillUnmount(): void;
    _onUpdate: (result: QueryResult, loaded: boolean, errors: ReadonlyArray<string>, errorDetails: ReadonlyArray<ErrorResult>) => void;
    _render(): {} | null | undefined;
    render(): any;
}
export declare function query<QueryResult>(query: Props<QueryResult>["query"], children: Props<QueryResult>["children"], options?: Pick<Props<QueryResult>, "renderLoading" | "renderErrors">): React.ComponentElement<Readonly<{
    children?: React.ReactNode;
}> & Readonly<Props<QueryResult>>, React.Component<Props<QueryResult>, React.ComponentState>>;
