"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var PropTypes = require("prop-types");
exports.PropTypes = PropTypes;
var not_equal_1 = require("bicycle/utils/not-equal");
var query_1 = require("bicycle/typed-helpers/query");
var client_shape_1 = require("../client-shape");
var errorStyle = {
    whiteSpace: "pre-wrap",
    fontFamily: "monospace",
    fontSize: "18px",
    padding: "9px",
    background: "#900000",
    color: "white"
};
function queryCache(client, query) {
    if (query instanceof query_1.BaseRootQuery) {
        return client.queryCache(query);
    }
    else {
        return client.queryCache(query);
    }
}
function subscribe(client, query, onUpdate) {
    if (query instanceof query_1.BaseRootQuery) {
        return client.subscribe(query, onUpdate);
    }
    else {
        return client.subscribe(query, onUpdate);
    }
}
var loadingComponents = [];
setInterval(function () {
    var now = Date.now();
    for (var i = 0; i < loadingComponents.length; i++) {
        var start = loadingComponents[i]._startLoadingTime;
        loadingComponents[i].setState({
            loadingDuration: start ? now - start : 0
        });
    }
}, 100);
var Connect = /** @class */ (function (_super) {
    __extends(Connect, _super);
    function Connect(props, context) {
        var _this = _super.call(this, props, context) || this;
        _this._previousElement = null;
        _this._startLoadingTime = null;
        _this._onUpdate = function (result, loaded, errors, errorDetails) {
            if (loaded) {
                if (_this._startLoadingTime !== null) {
                    _this._startLoadingTime = null;
                    var index = loadingComponents.indexOf(_this);
                    if (index !== -1) {
                        loadingComponents.splice(index, 1);
                    }
                }
            }
            else {
                if (_this._startLoadingTime === null) {
                    _this._startLoadingTime = Date.now();
                    loadingComponents.push(_this);
                }
            }
            _this.setState({
                result: result,
                loaded: loaded,
                errors: errors,
                errorDetails: errorDetails,
                loadingDuration: _this._startLoadingTime
                    ? Date.now() - _this._startLoadingTime
                    : 0
            });
        };
        _this._client = context.bicycleClient;
        if (!_this._client) {
            throw new Error("Could not find \"client\" in either the context or " +
                "props of Connect. " +
                "Either wrap the root component in a <Provider>, " +
                "or explicitly pass \"client\" as a prop to Connect.");
        }
        _this._query = _this.props.query;
        var _a = _this._query
            ? queryCache(_this._client, _this._query)
            : {
                result: undefined,
                loaded: true,
                errors: [],
                errorDetails: []
            }, result = _a.result, loaded = _a.loaded, errors = _a.errors, errorDetails = _a.errorDetails;
        _this.state = { result: result, loaded: loaded, errors: errors, errorDetails: errorDetails, loadingDuration: 0 };
        return _this;
    }
    Connect.prototype.componentDidMount = function () {
        if (this._query) {
            this._subscription = subscribe(this._client, this._query, this._onUpdate);
        }
    };
    Connect.prototype.componentWillReceiveProps = function (nextProps) {
        var newQuery = nextProps.query;
        if (!newQuery) {
            if (this._subscription) {
                this._subscription.unsubscribe();
            }
        }
        else if (!this._query ||
            not_equal_1.default(this._query instanceof query_1.BaseRootQuery ? this._query._query : this._query, newQuery instanceof query_1.BaseRootQuery ? newQuery._query : newQuery)) {
            if (this._subscription) {
                this._subscription.unsubscribe();
            }
            this._query = newQuery;
            if (this._query) {
                this._subscription = subscribe(this._client, this._query, this._onUpdate);
            }
        }
    };
    Connect.prototype.componentWillUnmount = function () {
        if (this._subscription) {
            this._subscription.unsubscribe();
        }
        var index = loadingComponents.indexOf(this);
        if (index !== -1) {
            loadingComponents.splice(index, 1);
        }
    };
    Connect.prototype._render = function () {
        if (this.state.errors.length && this.props.renderErrors !== true) {
            if (this.props.renderErrors === false) {
                return null;
            }
            else if (typeof this.props.renderErrors === "function") {
                return this.props.renderErrors(this.state.errors, this.context.client, __assign({}, this.state, { previousElement: this._previousElement }));
                // TODO: allow using context to set error renderer
            }
            else {
                return React.createElement("div", {}, this.state.errors.map(function (err, i) {
                    return React.createElement("div", { key: i, style: errorStyle }, err + "");
                }));
            }
        }
        if (!this.state.loaded && this.props.renderLoading !== true) {
            if (this.props.renderLoading === false) {
                return null;
            }
            else if (typeof this.props.renderLoading === "function") {
                return this.props.renderLoading(this.state.loadingDuration, this.context.client, __assign({}, this.state, { previousElement: this._previousElement }));
                // TODO: allow using context to set loading renderer
            }
            else {
                if (this.state.loadingDuration > 1000) {
                    return React.createElement("div", {}, "Loading...");
                }
                else {
                    return null;
                }
            }
        }
        return this.props.children(this.state.result, this.context.client, __assign({}, this.state, { previousElement: this._previousElement }));
    };
    Connect.prototype.render = function () {
        return (this._previousElement = this._render());
    };
    Connect.contextTypes = {
        bicycleClient: client_shape_1.default
    };
    return Connect;
}(React.Component));
exports.default = Connect;
function query(query, children, options) {
    var C = Connect;
    return React.createElement(C, __assign({ query: query,
        children: children }, options));
}
exports.query = query;
//# sourceMappingURL=connect.js.map