"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var connect_errors_1 = require("./connect-errors");
function createErrorBox(error, onDismiss) {
    var div = document.createElement('div');
    div.setAttribute('style', [
        'white-space: pre-wrap;',
        'font-family: monospace;',
        'font-size: 18px;',
        'padding: 9px;',
        'margin: 9px;',
        'background: #900000;',
        'color: white;',
        'z-index: 99999;',
    ].join(''));
    var msg = document.createElement('div');
    msg.textContent = error.message + '';
    div.appendChild(msg);
    var devWarning = document.createElement('div');
    devWarning.textContent = '(this error is only shown in development)';
    div.appendChild(devWarning);
    var dismissButton = document.createElement('button');
    dismissButton.setAttribute('style', [
        'margin: 9px 0 0 0;',
        'padding: 9px;',
        'border: 0;',
        'background: #ffffff;',
        'font-size: 100%;',
        'vertical-align: baseline;',
        'font-family: inherit;',
        'font-weight: inherit;',
        'color: black;',
    ].join(''));
    dismissButton.textContent = 'Dismiss';
    dismissButton.addEventListener('click', onDismiss.bind(null, error), false);
    div.appendChild(dismissButton);
    return div;
}
var Errors = /** @class */ (function (_super) {
    __extends(Errors, _super);
    function Errors() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Errors.prototype.componentDidMount = function () {
        this._renderErrors(this.props);
    };
    Errors.prototype.componentWillReceiveProps = function (newProps) {
        if (newProps.networkErrors !== this.props.networkErrors ||
            newProps.mutationErrors !== this.props.mutationErrors) {
            this._renderErrors(newProps);
        }
    };
    Errors.prototype._renderErrors = function (props) {
        if (this._errorContainer) {
            document.body.removeChild(this._errorContainer);
            this._errorContainer = undefined;
        }
        if (props.networkErrors.length || props.mutationErrors.length) {
            var errorContainer_1 = this._errorContainer = document.createElement('div');
            errorContainer_1.setAttribute('style', [
                'position: fixed;',
                'top: 0;',
                'left: 0;',
                'right: 0;',
                'bottom: 0;',
                'background: rgba(0, 0, 0, 0.8);',
                'z-index: 99999;',
            ].join(''));
            props.networkErrors.forEach(function (err) { return errorContainer_1.appendChild(createErrorBox(err, props.onDismissNetworkError)); });
            props.mutationErrors.forEach(function (err) { return errorContainer_1.appendChild(createErrorBox(err, props.onDismissMutationError)); });
            document.body.appendChild(this._errorContainer);
        }
    };
    Errors.prototype.render = function () {
        return React.Children.only(this.props.children);
    };
    return Errors;
}(React.Component));
exports.Errors = Errors;
exports.default = connect_errors_1.default()(Errors);
//# sourceMappingURL=default-error-renderer.js.map