"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var PropTypes = require("prop-types");
var client_shape_1 = require("../client-shape");
// prevent this code loading in production
var DefaultErrorRenderer;
if (process.env.NODE_ENV !== 'production') {
    DefaultErrorRenderer = require('./default-error-renderer').default;
}
var errorStyle = {
    whiteSpace: 'pre-wrap',
    fontFamily: 'monospace',
    fontSize: '18px',
    padding: '9px',
    background: '#900000',
    color: 'white',
};
var didWarnAboutReceivingClient = false;
function warnAboutReceivingClient() {
    if (didWarnAboutReceivingClient) {
        return;
    }
    didWarnAboutReceivingClient = true;
    console.error(// eslint-disable-line no-console
    '<Provider> does not support changing `client`, `renderLoading` or `renderErrors` on the fly.');
}
var defaultLoadingElement = React.createElement('div', {}, 'Loading...');
var Provider = /** @class */ (function (_super) {
    __extends(Provider, _super);
    function Provider(props, context) {
        var _this = _super.call(this, props, context) || this;
        _this.client = props.client;
        _this.renderLoading = props.renderLoading;
        _this.renderErrors = props.renderErrors;
        return _this;
    }
    Provider.prototype.getChildContext = function () {
        return {
            bicycleClient: this.client,
            bicycleRenderLoading: this.renderLoading,
            bicycleRenderErrors: this.renderErrors,
        };
    };
    Provider.prototype.componentWillReceiveProps = function (nextProps) {
        if (this.client !== nextProps.client ||
            this.renderLoading !== nextProps.renderLoading ||
            this.renderErrors !== nextProps.renderErrors) {
            warnAboutReceivingClient();
        }
    };
    Provider.prototype.render = function () {
        if (process.env.NODE_ENV !== 'production' && !this.props.disableErrorReporter) {
            return React.createElement(DefaultErrorRenderer, {}, this.props.children);
        }
        return React.Children.only(this.props.children);
    };
    Provider.propTypes = {
        client: client_shape_1.default.isRequired,
        disableErrorReporter: PropTypes.bool,
        renderLoading: PropTypes.oneOfType([
            PropTypes.any,
            PropTypes.bool,
        ]),
        renderErrors: PropTypes.oneOfType([
            PropTypes.any,
            PropTypes.bool,
        ]),
        children: PropTypes.element.isRequired,
    };
    Provider.defaultProps = {
        renderLoading: function (props) { return defaultLoadingElement; },
        renderErrors: function (props) { return React.createElement('div', {}, props.errors.map(function (err, i) {
            return React.createElement('div', { key: i, style: errorStyle }, err + '');
        })); },
    };
    Provider.childContextTypes = {
        bicycleClient: client_shape_1.default.isRequired,
        bicycleRenderLoading: PropTypes.oneOfType([
            PropTypes.any,
            PropTypes.bool,
        ]).isRequired,
        bicycleRenderErrors: PropTypes.oneOfType([
            PropTypes.any,
            PropTypes.bool,
        ]).isRequired,
    };
    return Provider;
}(React.Component));
exports.default = Provider;
//# sourceMappingURL=provider.js.map